const ICONS = {
	"default" : "../css/img/icon_group.png",
	"sketchup_item" : "../css/img/icon_scene.png",
}

$(document).ready(function() {
	$('#tree_view').jstree({
		'core' : {
			"animation" : 0,
			"check_callback" : true,
			"themes" : { "stripes" : true },
			"multiple" : true
		},
		"types" : {
			"#" : {
				"max_children" : 1,
				"max_depth" : 4,
				"valid_children" : [
					"default",
					"sketchup_item",
				]
			},
			"default" : {
				"icon" : ICONS["default"],
				"valid_children" : ["default", "sketchup_item"]
			},
			"sketchup_item" : {
				"icon" : ICONS["sketchup_item"],
				"valid_children" : []
			},
    },
		"plugins" : [
			"search", "contextmenu",
			"state", "types", "wholerow", 
			"checkbox"
		],
		"contextmenu": {
			"items": function () {
				return itemsCustomMenu()
			},
			"show_at_node": false
		},
		"checkbox": {
			"visible": false,
      // "three_state" : false, 
      "whole_node" : false,//Used to check/uncheck node only if clicked on checkbox icon, and not on the whole node including label
      "tie_selection" : false
    },
		"search": {
			"show_only_matches": true,
    },
	});

	$( window ).resize(function() {
		layoutDialog();
	});

	// search
	var to = false;
  $('#search').keyup(function () {
    if(to) { clearTimeout(to); }
    to = setTimeout(function () {
      var v = $('#search').val();
      $('#tree_view').jstree(true).search(v);
    }, 250);
	});

	treeEvents()

	sketchup.ready()
})

function layoutDialog() {
	var height = $( document ).height()
	var header_height = $( "#header" ).height()
	var footer_height = $( "#footer" ).height()

	tree_height = height-(header_height + footer_height)
	$('#tree_view').height(tree_height)
}

function setDataTree(data) {
	console.log(data)
	$('#tree_view').jstree(true).settings.core.data = data;
	$('#tree_view').jstree(true).load_node("#");
}

function toggleChecboxes() {
	$('#tree_view').jstree(true).toggle_checkboxes();
	selected = getAllSketchupItemSelected(undefined, false)
	sketchup.selection_changed(selected)
}

// filter select
function getSelected(full) {
	var inst = 	$('#tree_view').jstree(true);
	var checkbox = $('.jstree-checkbox').is(":visible")

	if (full != true){ full = false }

	if (checkbox){
		selected = inst.get_checked(full);
	}else{
		selected = inst.get_selected(full);
	}
	return selected
}

function getAllSketchupItemSelected(selected, full, type) {
	var inst = 	$('#tree_view').jstree(true);
	var result = []

	if (full != true){ full = false }
	if (selected === undefined){ selected = getSelected(full) }

	for (let index = 0; index < selected.length; index++) {
		const element = selected[index];
		getAllSketchupItem(inst, element, result, full, type);
	}
	return result
}
function getAllSketchupItem(inst, nodeId, result, full, type) {
	var node = inst.get_node(nodeId);
	if (full === undefined){ full = true }

	if (full == true){
		value = node
	}else{
		value = node["id"]
	}
	if (type === undefined){
		if (!result.includes(value)){
			result.push(value);
		}
	}else{
		node_type = inst.get_type(node)
		if (node_type == type && !result.includes(value)){
			result.push(value);
		}
	}
	if (node.children) {
		for (var i = 0; i < node.children.length; i++) {
			getAllSketchupItem(inst, node.children[i], result, full);
		}
	}
}

function treeEvents() {
	$('#tree_view')
	.on("ready.jstree", function (e, data) {
		// sketchup.tree_view_ready()
	})
	
	// refresh
	.on('refresh.jstree', function (e, data) {
	})

	// Selection
	.on("changed.jstree", function (e, data) {
		var checkbox = $('.jstree-checkbox').is(":visible")
		if (!checkbox){ 
			var inst = 	$('#tree_view').jstree(true);
			selected = inst.get_selected(true);
			sketchup.selection_changed(getAllSketchupItemSelected(selected, false))
		}
	})
	// checkbox
	.on('check_node.jstree', function (e, data) {
		selected = $('#tree_view').jstree(true).get_checked(true);
		sketchup.selection_changed(getAllSketchupItemSelected(selected, false))
	})
	.on('uncheck_node.jstree', function (e, data) {
		selected = $('#tree_view').jstree(true).get_checked(true);
		sketchup.selection_changed(getAllSketchupItemSelected(selected, false))
	})
	.on('check_all.jstree', function (e, data) {
		selected = $('#tree_view').jstree(true).get_checked(true);
		sketchup.selection_changed(getAllSketchupItemSelected(selected, false))
	})
	.on('uncheck_all.jstree', function (e, data) {
		selected = $('#tree_view').jstree(true).get_checked(true);
		sketchup.selection_changed(getAllSketchupItemSelected(selected, false))
	})

	.on('search.jstree', function (e, data) {
		console.log("search.jstree", $("#search").val())
	})
}

function itemsCustomMenu() {
	var items = {
		"select_all" : {
			"separator_before"	: false,
			"separator_after"	: false,
			"_disabled"			: false,
			"label"				: function () {
				if ($('.jstree-checkbox').is(":visible")){
					return "Check All"
				}else{
					return "Select all"
				}
			},
			"action"			: function (data) {
				var inst = $('#tree_view').jstree(true);
				if ($('.jstree-checkbox').is(":visible")){
					inst.check_all();
				}else{
					inst.select_all();
				}
			}
		},
		"uncheck_all" : {
			"separator_before"	: false,
			"separator_after"	: true,
			"_disabled"			: false,
			"label"				: function () {
				if ($('.jstree-checkbox').is(":visible")){
					return "Uncheck All"
				}else{
					return "Deselect all"
				}
			},
			"action"			: function (data) {
				var inst = $('#tree_view').jstree(true);
				if ($('.jstree-checkbox').is(":visible")){
					inst.uncheck_all();
				}else{
					inst.deselect_all();
				}
			}
		},
	};
	return items;
}